@@ -1,15 +1,15 @@
 """Some utilities for manipulating GeoSpatial data."""
+from __future__ import annotations
+
 import contextlib
-import logging
-import sys
 import tempfile
 import uuid
 from dataclasses import dataclass
-from numbers import Number
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Sequence, Tuple, TypeVar, Union
+from typing import Any, Sequence, Tuple, TypeVar, Union
 
 import cytoolz as tlz
+import dask
 import geopandas as gpd
 import numpy as np
 import pandas as pd
@@ -37,18 +37,12 @@ from .exceptions import (
     UnprojectedCRSError,
 )
 
-logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
-handler = logging.StreamHandler(sys.stdout)
-handler.setFormatter(logging.Formatter(""))
-logger.handlers = [handler]
-logger.propagate = False
-
-DEF_CRS = "epsg:4326"
 BOX_ORD = "(west, south, east, north)"
 GTYPE = Union[Polygon, MultiPolygon, Tuple[float, float, float, float]]
 GDF = TypeVar("GDF", gpd.GeoDataFrame, gpd.GeoSeries)
 XD = TypeVar("XD", xr.Dataset, xr.DataArray)
+CRSTYPE = Union[int, str, pyproj.CRS]
+NUMBER = Union[int, float, np.number]
 
 __all__ = [
     "snap2nearest",
@@ -66,9 +60,7 @@ __all__ = [
 ]
 
 
-def arcgis2geojson(
-    arcgis: Union[str, Dict[str, Any]], id_attr: Optional[str] = None
-) -> Dict[str, Any]:
+def arcgis2geojson(arcgis: str | dict[str, Any], id_attr: str | None = None) -> dict[str, Any]:
     """Convert ESRIGeoJSON format to GeoJSON.
 
     Notes
@@ -94,9 +86,9 @@ def arcgis2geojson(
 
 
 def json2geodf(
-    content: Union[List[Dict[str, Any]], Dict[str, Any]],
-    in_crs: Union[str, pyproj.CRS] = DEF_CRS,
-    crs: Union[str, pyproj.CRS] = DEF_CRS,
+    content: list[dict[str, Any]] | dict[str, Any],
+    in_crs: CRSTYPE = 4326,
+    crs: CRSTYPE = 4326,
 ) -> gpd.GeoDataFrame:
     """Create GeoDataFrame from (Geo)JSON.
 
@@ -104,9 +96,9 @@ def json2geodf(
     ----------
     content : dict or list of dict
         A (Geo)JSON dictionary e.g., response.json() or a list of them.
-    in_crs: str or pyproj.CRS
+    in_crs : int, str, or pyproj.CRS, optional
         CRS of the content, defaults to ``epsg:4326``.
-    crs: str or pyproj.CRS, optional
+    crs : int, str, or pyproj.CRS, optional
         The target CRS of the output GeoDataFrame, defaults to ``epsg:4326``.
 
     Returns
@@ -139,8 +131,8 @@ def json2geodf(
 
 def xarray_geomask(
     ds: XD,
-    geometry: Union[Polygon, MultiPolygon],
-    crs: Union[str, pyproj.CRS],
+    geometry: Polygon | MultiPolygon,
+    crs: CRSTYPE,
     all_touched: bool = False,
     drop: bool = True,
     from_disk: bool = False,
@@ -153,7 +145,7 @@ def xarray_geomask(
         The dataset(array) to be masked
     geometry : Polygon, MultiPolygon
         The geometry to mask the data
-    crs: str or pyproj.CRS
+    crs : int, str, or pyproj.CRS
         The spatial reference of the input geometry
     all_touched : bool, optional
         Include a pixel in the mask if it touches any of the shapes.
@@ -188,9 +180,9 @@ def xarray_geomask(
 
 def get_gtiff_attrs(
     resp: bytes,
-    ds_dims: Optional[Tuple[str, str]] = None,
-    driver: Optional[str] = None,
-    nodata: Optional[Number] = None,
+    ds_dims: tuple[str, str] | None = None,
+    driver: str | None = None,
+    nodata: NUMBER | None = None,
 ) -> Attrs:
     """Get nodata, CRS, and dimension names in (vertical, horizontal) order from raster in bytes.
 
@@ -235,15 +227,15 @@ def get_gtiff_attrs(
 
 
 def gtiff2xarray(
-    r_dict: Dict[str, bytes],
-    geometry: Optional[GTYPE] = None,
-    geo_crs: Optional[str] = None,
-    ds_dims: Optional[Tuple[str, str]] = None,
-    driver: Optional[str] = None,
+    r_dict: dict[str, bytes],
+    geometry: GTYPE | None = None,
+    geo_crs: CRSTYPE | None = None,
+    ds_dims: tuple[str, str] | None = None,
+    driver: str | None = None,
     all_touched: bool = False,
-    nodata: Optional[Number] = None,
+    nodata: NUMBER | None = None,
     drop: bool = True,
-) -> Union[xr.DataArray, xr.Dataset]:
+) -> xr.DataArray | xr.Dataset:
     """Convert (Geo)Tiff byte responses to ``xarray.Dataset``.
 
     Parameters
@@ -254,7 +246,7 @@ def gtiff2xarray(
     geometry : Polygon, MultiPolygon, or tuple, optional
         The geometry to mask the data that should be in the same CRS as the r_dict.
         Defaults to ``None``.
-    geo_crs: str or pyproj.CRS, optional
+    geo_crs : int, str, or pyproj.CRS, optional
         The spatial reference of the input geometry, defaults to ``None``. This
         argument should be given when ``geometry`` is given.
     ds_dims : tuple of str, optional
@@ -293,8 +285,8 @@ def gtiff2xarray(
         var_name = {lyr: "_".join(lyr.split("_")[:-3]) for lyr in r_dict.keys()}
 
     attrs = get_gtiff_attrs(r_dict[key1], ds_dims, driver, nodata)
-    dtypes: Dict[str, type] = {}
-    nodata_dict: Dict[str, Number] = {}
+    dtypes: dict[str, type] = {}
+    nodata_dict: dict[str, NUMBER] = {}
 
     tmp_dir = tempfile.gettempdir()
 
@@ -305,7 +297,6 @@ def gtiff2xarray(
                 ds = rxr.open_rasterio(vrt)
                 with contextlib.suppress(ValueError):
                     ds = ds.squeeze("band", drop=True)
-                ds = ds.sortby(attrs.dims[0], ascending=False)
                 ds.name = var_name[lyr]
                 dtypes[ds.name] = ds.dtype
                 nodata_dict[ds.name] = utils.get_nodata(vrt) if nodata is None else nodata
@@ -314,33 +305,31 @@ def gtiff2xarray(
                 ds.to_netcdf(fpath)
                 return fpath
 
-    ds = xr.open_mfdataset(
-        (to_dataset(lyr, resp) for lyr, resp in r_dict.items()),
-        parallel=True,
-        decode_coords="all",
-    )
-    ds = ds.sortby(attrs.dims[0], ascending=False)
+    with dask.config.set(**{"array.slicing.split_large_chunks": True}):
+        ds = xr.open_mfdataset(
+            (to_dataset(lyr, resp) for lyr, resp in r_dict.items()),
+            chunks="auto",
+            parallel=True,
+            engine="rasterio",
+        )
+        ds = ds.squeeze("band", drop=True)
 
-    variables = list(ds)
-    _gm = ds.rio.get_gcps()
-    grid_mapping = _gm.grid_mapping if _gm else "spatial_ref"
-    if grid_mapping in ds:
-        ds = ds.drop_vars(grid_mapping)
+        variables = list(ds)
+
+        if len(variables) == 1:
+            ds = ds[variables[0]].copy()
+            ds = ds.astype(dtypes[variables[0]])
+            ds.attrs["crs"] = attrs.crs.to_string()
+            ds.attrs["nodatavals"] = (nodata_dict[ds.name],)
+            ds = ds.rio.write_nodata(nodata_dict[ds.name])
+        else:
+            ds.attrs["crs"] = attrs.crs.to_string()
+            for v in variables:
+                ds[v] = ds[v].astype(dtypes[v])
+                ds[v].attrs["crs"] = attrs.crs.to_string()
+                ds[v].attrs["nodatavals"] = (nodata_dict[v],)
+                ds[v] = ds[v].rio.write_nodata(nodata_dict[v])
 
-    if len(variables) == 1:
-        ds = ds[variables[0]].copy()
-        ds = ds.astype(dtypes[variables[0]])
-        ds.attrs["crs"] = attrs.crs.to_string()
-        ds.attrs["nodatavals"] = (nodata_dict[ds.name],)
-        ds = ds.rio.write_nodata(nodata_dict[ds.name])
-    else:
-        ds.attrs["crs"] = attrs.crs.to_string()
-        for v in variables:
-            ds[v] = ds[v].astype(dtypes[v])
-            ds[v].attrs["crs"] = attrs.crs.to_string()
-            ds[v].attrs["nodatavals"] = (nodata_dict[v],)
-            ds[v] = ds[v].rio.write_nodata(nodata_dict[v])
-    ds = ds.rio.write_crs(attrs.crs, grid_mapping_name=grid_mapping)
     if isinstance(geometry, (Polygon, MultiPolygon)):
         if geo_crs is None:
             raise MissingCRSError
@@ -349,9 +338,9 @@ def gtiff2xarray(
 
 
 def get_transform(
-    ds: Union[xr.Dataset, xr.DataArray],
-    ds_dims: Tuple[str, str] = ("y", "x"),
-) -> Tuple[rio.Affine, int, int]:
+    ds: xr.Dataset | xr.DataArray,
+    ds_dims: tuple[str, str] = ("y", "x"),
+) -> tuple[rio.Affine, int, int]:
     """Get transform of a ``xarray.Dataset`` or ``xarray.DataArray``.
 
     Parameters
@@ -386,8 +375,8 @@ def get_transform(
 
 def geo2polygon(
     geometry: GTYPE,
-    geo_crs: Union[str, pyproj.CRS],
-    crs: Union[str, pyproj.CRS],
+    geo_crs: CRSTYPE,
+    crs: CRSTYPE,
 ) -> Polygon:
     """Convert a geometry to a Shapely's Polygon and transform to any CRS.
 
@@ -395,9 +384,9 @@ def geo2polygon(
     ----------
     geometry : Polygon or tuple of length 4
         Polygon or bounding box (west, south, east, north).
-    geo_crs: str or pyproj.CRS
+    geo_crs : int, str, or pyproj.CRS
         Spatial reference of the input geometry
-    crs: str or pyproj.CRS
+    crs : int, str, or pyproj.CRS
         Target spatial reference.
 
     Returns
@@ -420,7 +409,7 @@ def geo2polygon(
 
 
 def xarray2geodf(
-    da: xr.DataArray, dtype: str, mask_da: Optional[xr.DataArray] = None, connectivity: int = 8
+    da: xr.DataArray, dtype: str, mask_da: xr.DataArray | None = None, connectivity: int = 8
 ) -> gpd.GeoDataFrame:
     """Vectorize a ``xarray.DataArray`` to a ``geopandas.GeoDataFrame``.
 
@@ -482,6 +471,9 @@ class Coordinates:
         Longitude(s) in decimal degrees.
     lat : float or list of floats
         Latitude(s) in decimal degrees.
+    bounds : tuple of length 4, optional
+        The bounding box to check of the input coordinates fall within.
+        Defaults to WGS84 bounds.
 
     Examples
     --------
@@ -491,12 +483,12 @@ class Coordinates:
     [100.0, -30.0]
     """
 
-    lon: Union[Number, Sequence[Number]]
-    lat: Union[Number, Sequence[Number]]
-    bounds: Optional[Tuple[float, float, float, float]] = None
+    lon: NUMBER | Sequence[NUMBER]
+    lat: NUMBER | Sequence[NUMBER]
+    bounds: tuple[float, float, float, float] | None = None
 
     @staticmethod
-    def __box_geo(bounds: Optional[Tuple[float, float, float, float]]) -> sgeom.Polygon:
+    def __box_geo(bounds: tuple[float, float, float, float] | None) -> sgeom.Polygon:
         """Get EPSG:4326 CRS."""
         if bounds is None:
             return sgeom.box(*pyproj.CRS(4326).area_of_use.bounds)
@@ -516,13 +508,11 @@ class Coordinates:
 
     def __post_init__(self) -> None:
         """Normalize the longitude value within [-180, 180)."""
-        _lon = [self.lon] if isinstance(self.lon, Number) else self.lon
-        lat = [self.lat] if isinstance(self.lat, Number) else self.lat
-        if not isinstance(_lon, (list, tuple)) and not isinstance(lat, (list, tuple)):
-            raise InputTypeError("lon/lat", "float or list")
+        _lon = [self.lon] if isinstance(self.lon, (int, float)) else list(self.lon)
+        lat = [self.lat] if isinstance(self.lat, (int, float)) else list(self.lat)
 
-        lon = np.mod(np.mod(_lon, 360) + 540, 360) - 180
-        pts = gpd.GeoSeries([sgeom.Point(xy) for xy in zip(lon, lat)], crs="epsg:4326")
+        lon = np.mod(np.mod(_lon, 360.0) + 540.0, 360.0) - 180.0
+        pts = gpd.GeoSeries([sgeom.Point(xy) for xy in zip(lon, lat)], crs=4326)
         self._points = self.__validate(pts, self.__box_geo(self.bounds))
 
     @property
@@ -531,12 +521,12 @@ class Coordinates:
         return self._points
 
 
-def validate_crs(val: Union[str, int, pyproj.CRS]) -> str:
+def validate_crs(crs: CRSTYPE) -> str:
     """Validate a CRS.
 
     Parameters
     ----------
-    val : str or int
+    crs : str, int, or pyproj.CRS
         Input CRS.
 
     Returns
@@ -545,10 +535,9 @@ def validate_crs(val: Union[str, int, pyproj.CRS]) -> str:
         Validated CRS as a string.
     """
     try:
-        crs: str = pyproj.CRS(val).to_string()
+        return pyproj.CRS(crs).to_string()  # type: ignore
     except pyproj.exceptions.CRSError as ex:
         raise InputTypeError("crs", "a valid CRS") from ex
-    return crs
 
 
 @dataclass
@@ -601,7 +590,7 @@ class GeoBSpline:
     ...         (-97.06127, 32.832),
     ...     ]
     ... )
-    >>> pts = gpd.GeoSeries(gpd.points_from_xy(xl, yl, crs="epsg:4326"))
+    >>> pts = gpd.GeoSeries(gpd.points_from_xy(xl, yl, crs=4326))
     >>> sp = GeoBSpline(pts.to_crs("epsg:3857"), 5).spline
     >>> pts_sp = gpd.GeoSeries(gpd.points_from_xy(sp.x, sp.y, crs="epsg:3857"))
     >>> pts_sp = pts_sp.to_crs("epsg:4326")
@@ -615,8 +604,8 @@ class GeoBSpline:
 
     @staticmethod
     def __curvature(
-        xs: Union[Sequence[float], np.ndarray], ys: Union[Sequence[float], np.ndarray], l_tot: float
-    ) -> Tuple[np.ndarray, np.ndarray]:
+        xs: Sequence[float] | np.ndarray, ys: Sequence[float] | np.ndarray, l_tot: float
+    ) -> tuple[np.ndarray, np.ndarray]:
         """Compute the curvature of a B-spline curve.
 
         Notes
@@ -846,8 +835,8 @@ def break_lines(lines: GDF, points: gpd.GeoDataFrame, tol: float = 0.0) -> GDF:
 
 
 def geometry_list(
-    geometry: Union[GTYPE, sgeom.Point, sgeom.MultiPoint, sgeom.LineString, sgeom.MultiLineString]
-) -> List[Union[sgeom.Polygon, sgeom.Point, sgeom.LineString]]:
+    geometry: GTYPE | sgeom.Point | sgeom.MultiPoint | sgeom.LineString | sgeom.MultiLineString,
+) -> list[sgeom.Polygon | sgeom.Point | sgeom.LineString]:
     """Get a list of polygons, points, and lines from a geometry."""
     if isinstance(geometry, (sgeom.Polygon, sgeom.LineString, sgeom.Point)):
         return [geometry]
